@@ -48,6 +48,8 @@ void totem_playlist_add_files (GtkWidget *widget, TotemPlaylist *playlist);
 void playlist_remove_button_clicked (GtkWidget *button, TotemPlaylist *playlist);
 void totem_playlist_up_files (GtkWidget *widget, TotemPlaylist *playlist);
 void totem_playlist_down_files (GtkWidget *widget, TotemPlaylist *playlist);
+void playlist_copy_location_action_callback (GtkAction *action, TotemPlaylist *playlist);
+void playlist_remove_action_callback (GtkAction *action, TotemPlaylist *playlist);
 
 typedef gboolean (*PlaylistCallback) (TotemPlaylist *playlist, const char *mrl,
 		gpointer data);
@@ -89,7 +91,9 @@ struct TotemPlaylistPrivate
 
 	GConfClient *gc;
 
-	int x, y;
+	/* Used to know the position for drops */
+	GtkTreePath *tree_path;
+	GtkTreeViewDropPosition drop_pos;
 
 	guint disable_save_to_disk : 1;
 
@@ -142,14 +146,6 @@ static const GtkTargetEntry target_table[] = {
 	{ "_NETSCAPE_URL", 0, 1 },
 };
 
-static void playlist_remove_action_activated (GtkAction *action, TotemPlaylist *playlist);
-static void playlist_copy_location_action_activated (GtkAction *action, TotemPlaylist *playlist);
-
-static const GtkActionEntry entries[] = {
-	{ "remove", GTK_STOCK_REMOVE, N_("_Remove"), NULL, N_("Remove file from playlist"), G_CALLBACK (playlist_remove_action_activated) },
-	{ "copy-location", GTK_STOCK_COPY, N_("_Copy Location"), NULL, N_("Copy the location to the clipboard"), G_CALLBACK (playlist_copy_location_action_activated) }
-};
-
 static void totem_playlist_class_init (TotemPlaylistClass *class);
 static void totem_playlist_init       (TotemPlaylist      *playlist);
 
@@ -357,14 +353,14 @@ gtk_tree_selection_has_selected (GtkTreeSelection *selection)
 }
 
 static void
-drop_cb (GtkWidget     *widget,
-         GdkDragContext     *context, 
-	 gint                x,
-	 gint                y,
-	 GtkSelectionData   *data, 
-	 guint               info, 
-	 guint               time, 
-	 TotemPlaylist        *playlist)
+drop_cb (GtkWidget        *widget,
+         GdkDragContext   *context, 
+	 gint              x,
+	 gint              y,
+	 GtkSelectionData *data, 
+	 guint             info, 
+	 guint             time, 
+	 TotemPlaylist    *playlist)
 {
 	GList *list, *p, *file_list;
 
@@ -378,27 +374,36 @@ drop_cb (GtkWidget     *widget,
 	p = list;
 	file_list = NULL;
 
-	while (p != NULL)
-	{
+	while (p != NULL) {
+		/* We get the list in the wrong order here,
+		 * so when we insert the files at the same position
+		 * in the tree, they are in the right order.*/
 		file_list = g_list_prepend (file_list,
-				gnome_vfs_uri_to_string
-				((const GnomeVFSURI*)(p->data), 0));
+					    gnome_vfs_uri_to_string ((const GnomeVFSURI*)(p->data), 0));
 		p = p->next;
 	}
 
 	gnome_vfs_uri_list_free (list);
-	file_list = g_list_reverse (file_list);
 
-	if (file_list == NULL)
-	{
+	if (file_list == NULL) {
 		gtk_drag_finish (context, FALSE, FALSE, time);
 		return;
 	}
 
 	totem_playlist_set_waiting_cursor (playlist);
 
-	for (p = file_list; p != NULL; p = p->next)
-	{
+	playlist->_priv->tree_path = gtk_tree_path_new ();
+	gtk_tree_view_get_dest_row_at_pos (GTK_TREE_VIEW (playlist->_priv->treeview),
+					   x, y,
+					   &playlist->_priv->tree_path,
+					   &playlist->_priv->drop_pos);
+
+	/* But we reverse the list if we don't have any items in the
+	 * list, as we insert new items at the end */
+	if (playlist->_priv->tree_path == NULL)
+		file_list = g_list_reverse (file_list);
+
+	for (p = file_list; p != NULL; p = p->next) {
 		char *filename, *title;
 
 		if (p->data == NULL)
@@ -428,6 +433,8 @@ drop_cb (GtkWidget     *widget,
 
 	g_list_free (file_list);
 	gtk_drag_finish (context, TRUE, FALSE, time);
+	gtk_tree_path_free (playlist->_priv->tree_path);
+	playlist->_priv->tree_path = NULL;
 
 	totem_playlist_unset_waiting_cursor (playlist);
 
@@ -436,8 +443,8 @@ drop_cb (GtkWidget     *widget,
 			NULL);
 }
 
-static void
-playlist_copy_location_action_activated (GtkAction *action, TotemPlaylist *playlist)
+void
+playlist_copy_location_action_callback (GtkAction *action, TotemPlaylist *playlist)
 {
 	GList *l;
 	GtkTreePath *path;
@@ -609,9 +616,9 @@ button_release_cb (GtkWidget *treeview, GdkEventButton *event, gpointer data)
 	{
 		playlist->_priv->drop_disabled = FALSE;
 		totem_playlist_set_reorderable (playlist, FALSE);
-		gtk_drag_dest_set (treeview, GTK_DEST_DEFAULT_ALL,
-				target_table, G_N_ELEMENTS (target_table),
-				GDK_ACTION_COPY | GDK_ACTION_MOVE);
+		gtk_tree_view_enable_model_drag_dest (GTK_TREE_VIEW (treeview),
+						      target_table, G_N_ELEMENTS (target_table),
+						      GDK_ACTION_COPY | GDK_ACTION_MOVE);
 
 		g_signal_handlers_unblock_by_func (treeview,
 				(GFunc) drop_cb, data);
@@ -638,9 +645,10 @@ drag_end_cb (GtkWidget *treeview, GdkDragContext *context, gpointer data)
 	playlist->_priv->drop_disabled = FALSE;
 	playlist->_priv->drag_started = FALSE;
 	totem_playlist_set_reorderable (playlist, FALSE);
-	gtk_drag_dest_set (treeview, GTK_DEST_DEFAULT_ALL, target_table,
-        		G_N_ELEMENTS (target_table),
-			GDK_ACTION_COPY | GDK_ACTION_MOVE);
+
+	gtk_tree_view_enable_model_drag_dest (GTK_TREE_VIEW (treeview),
+					      target_table, G_N_ELEMENTS (target_table),
+					      GDK_ACTION_COPY | GDK_ACTION_MOVE);
 
 	g_signal_handlers_unblock_by_func (treeview, (GFunc) drop_cb, data);
 
@@ -851,8 +859,8 @@ playlist_remove_button_clicked (GtkWidget *button, TotemPlaylist *playlist)
 	playlist_remove_files (playlist);
 }
 
-static void
-playlist_remove_action_activated (GtkAction *action, TotemPlaylist *playlist)
+void
+playlist_remove_action_callback (GtkAction *action, TotemPlaylist *playlist)
 {
 	playlist_remove_files (playlist);
 }
@@ -1183,24 +1191,21 @@ treeview_row_changed (GtkTreeView *treeview, GtkTreePath *arg1,
 		GtkTreeViewColumn *arg2, TotemPlaylist *playlist)
 {
 	if (totem_playlist_gtk_tree_path_equals
-			(arg1, playlist->_priv->current) != FALSE)
-	{
+	    (arg1, playlist->_priv->current) != FALSE) {
 		g_signal_emit (G_OBJECT (playlist),
 				totem_playlist_table_signals[ITEM_ACTIVATED], 0,
 				NULL);
 		return;
 	}
 
-	if (playlist->_priv->current != NULL)
-	{
+	if (playlist->_priv->current != NULL) {
 		totem_playlist_unset_playing (playlist);
 		gtk_tree_path_free (playlist->_priv->current);
 	}
 
 	playlist->_priv->current = gtk_tree_path_copy (arg1);
 
-	if (playlist->_priv->shuffle != FALSE)
-	{
+	if (playlist->_priv->shuffle != FALSE) {
 		int *indices, indice, i;
 
 		indices = gtk_tree_path_get_indices (playlist->_priv->current);
@@ -1222,9 +1227,10 @@ treeview_row_changed (GtkTreeView *treeview, GtkTreePath *arg1,
 	if (playlist->_priv->drop_disabled) {
 		playlist->_priv->drop_disabled = FALSE;
 		totem_playlist_set_reorderable (playlist, FALSE);
-		gtk_drag_dest_set (GTK_WIDGET (treeview), GTK_DEST_DEFAULT_ALL,
-				target_table, G_N_ELEMENTS (target_table),
-				GDK_ACTION_COPY | GDK_ACTION_MOVE);
+
+		gtk_tree_view_enable_model_drag_dest (GTK_TREE_VIEW (treeview),
+						      target_table, G_N_ELEMENTS (target_table),
+						      GDK_ACTION_COPY | GDK_ACTION_MOVE);
 
 		g_signal_handlers_unblock_by_func (treeview,
 				(GFunc) drop_cb, playlist);
@@ -1277,9 +1283,10 @@ init_treeview (GtkWidget *treeview, TotemPlaylist *playlist)
                         G_CALLBACK (drag_begin_cb), playlist);
 	g_signal_connect (G_OBJECT (treeview), "drag_end",
                         G_CALLBACK (drag_end_cb), playlist);
-	gtk_drag_dest_set (treeview, GTK_DEST_DEFAULT_ALL,
-			target_table, G_N_ELEMENTS (target_table),
-			GDK_ACTION_COPY | GDK_ACTION_MOVE);
+
+	gtk_tree_view_enable_model_drag_dest (GTK_TREE_VIEW (treeview),
+					      target_table, G_N_ELEMENTS (target_table),
+					      GDK_ACTION_COPY | GDK_ACTION_MOVE);
 
 	playlist->_priv->selection = selection;
 
@@ -1436,39 +1443,6 @@ init_config (TotemPlaylist *playlist)
 			GCONF_PREFIX"/shuffle", NULL) != FALSE;
 }
 
-/* FIXME this should live in the playlist parser */
-static int
-totem_embedded_parse_duration (const char *duration)
-{
-	int hours, minutes, seconds, fractions;
-
-	if (!duration)
-		return -1;
-
-	/* Formats used by both ASX and RAM files */
-	if (sscanf (duration, "%d:%d:%d.%d", &hours, &minutes, &seconds, &fractions) == 4) {
-		int ret = hours * 3600 + minutes * 60 + seconds;
-		if (ret == 0 && fractions > 0)
-			ret = 1;
-		return ret;
-	}
-	if (sscanf (duration, "%d:%d:%d", &hours, &minutes, &seconds) == 3)
-		return hours * 3600 + minutes * 60 + seconds;
-	if (sscanf (duration, "%d:%d.%d", &minutes, &seconds, &fractions) == 3) {
-		int ret = minutes * 60 + seconds;
-		if (ret == 0 && fractions > 0)
-			ret = 1;
-		return ret;
-	}
-	if (sscanf (duration, "%d:%d", &minutes, &seconds) == 2)
-		return minutes * 60 + seconds;
-	/* PLS files format */
-	if (sscanf (duration, "%d", &seconds) == 1)
-		return seconds;
-
-	return -1;
-}
-
 static void
 totem_playlist_entry_parsed (TotemPlParser *parser,
 			     const char *uri,
@@ -1479,8 +1453,8 @@ totem_playlist_entry_parsed (TotemPlParser *parser,
 	gint64 duration;
 
 	/* We ignore 0-length items in playlists, they're usually just banners */
-	duration = totem_embedded_parse_duration
-		(g_hash_table_lookup (metadata, TOTEM_PL_PARSER_FIELD_DURATION));
+	duration = totem_pl_parser_parse_duration
+		(g_hash_table_lookup (metadata, TOTEM_PL_PARSER_FIELD_DURATION), FALSE);
 	if (duration == 0)
 		return;
 	title = g_hash_table_lookup (metadata, TOTEM_PL_PARSER_FIELD_TITLE);
@@ -1511,7 +1485,6 @@ static void
 totem_playlist_init (TotemPlaylist *playlist)
 {
 	GtkWidget *container;
-	char *filename;
 
 	playlist->_priv = g_new0 (TotemPlaylistPrivate, 1);
 	playlist->_priv->parser = totem_pl_parser_new ();
@@ -1533,23 +1506,8 @@ totem_playlist_init (TotemPlaylist *playlist)
 		return;
 
 	/* popup menu */
-	playlist->_priv->action_group = 
-		gtk_action_group_new ("playlist-action-group");
-	gtk_action_group_set_translation_domain (playlist->_priv->action_group,
-			GETTEXT_PACKAGE);
-	gtk_action_group_add_actions (playlist->_priv->action_group, entries,
-			G_N_ELEMENTS (entries), playlist);
-
-	playlist->_priv->ui_manager = gtk_ui_manager_new ();
-	gtk_ui_manager_insert_action_group (playlist->_priv->ui_manager,
-			playlist->_priv->action_group, -1);
-
-	filename = totem_interface_get_full_path ("playlist-ui.xml");
-	if (!gtk_ui_manager_add_ui_from_file (playlist->_priv->ui_manager, filename, NULL)) {
-		totem_playlist_finalize (G_OBJECT (playlist));
-		return;
-	}
-	g_free (filename);
+	playlist->_priv->action_group = GTK_ACTION_GROUP (gtk_builder_get_object (playlist->_priv->xml, "playlist-action-group"));
+	playlist->_priv->ui_manager = GTK_UI_MANAGER (gtk_builder_get_object (playlist->_priv->xml, "totem-playlist-ui-manager"));
 
 	gtk_widget_add_events (GTK_WIDGET (playlist), GDK_KEY_PRESS_MASK);
 	g_signal_connect (G_OBJECT (playlist), "key_press_event",
@@ -1600,24 +1558,34 @@ totem_playlist_add_one_mrl (TotemPlaylist *playlist, const char *mrl,
 	GtkListStore *store;
 	GtkTreeIter iter;
 	char *filename_for_display, *uri;
+	GtkTreeRowReference *ref;
+	int pos;
 
 	g_return_val_if_fail (TOTEM_IS_PLAYLIST (playlist), FALSE);
 	g_return_val_if_fail (mrl != NULL, FALSE);
 
 	if (display_name == NULL)
-	{
 		filename_for_display = totem_playlist_mrl_to_title (mrl);
-	} else {
+	else
 		filename_for_display = g_strdup (display_name);
-	}
 
+	ref = NULL;
 	uri = totem_create_full_path (mrl);
 
 	D("totem_playlist_add_one_mrl (): %s %s %s\n",
 				filename_for_display, uri ? uri : "(null)", display_name);
 
+	if (playlist->_priv->tree_path != NULL && playlist->_priv->current != NULL) {
+		int *indices;
+		indices = gtk_tree_path_get_indices (playlist->_priv->tree_path);
+		pos = indices[0];
+		ref = gtk_tree_row_reference_new (playlist->_priv->model, playlist->_priv->current);
+	} else {
+		pos = G_MAXINT;
+	}
+
 	store = GTK_LIST_STORE (playlist->_priv->model);
-	gtk_list_store_insert_with_values (store, &iter, G_MAXINT32,
+	gtk_list_store_insert_with_values (store, &iter, pos,
 			PLAYING_COL, TOTEM_PLAYLIST_STATUS_NONE,
 			FILENAME_COL, filename_for_display,
 			URI_COL, uri ? uri : mrl,
@@ -1627,12 +1595,17 @@ totem_playlist_add_one_mrl (TotemPlaylist *playlist, const char *mrl,
 	g_free (filename_for_display);
 	g_free (uri);
 
-	if (playlist->_priv->current == NULL
-			&& playlist->_priv->shuffle == FALSE)
-		playlist->_priv->current = gtk_tree_model_get_path
-			(playlist->_priv->model, &iter);
+	if (playlist->_priv->current == NULL && playlist->_priv->shuffle == FALSE)
+		playlist->_priv->current = gtk_tree_model_get_path (playlist->_priv->model, &iter);
 	ensure_shuffled (playlist, playlist->_priv->shuffle);
 
+	/* And update current to point to the right file again */
+	if (ref != NULL) {
+		gtk_tree_path_free (playlist->_priv->current);
+		playlist->_priv->current = gtk_tree_row_reference_get_path (ref);
+		gtk_tree_row_reference_free (ref);
+	}
+
 	g_signal_emit (G_OBJECT (playlist),
 			totem_playlist_table_signals[CHANGED], 0,
 			NULL);
@@ -1777,59 +1750,36 @@ totem_playlist_clear_with_compare (TotemPlaylist *playlist, GCompareFunc func,
 	}
 }
 
-static int
-totem_playlist_compare_with_prefix (gconstpointer a, gconstpointer b)
-{
-	const char *mrl = (const char *) a;
-	const char *prefix = (const char *) b;
-
-	return g_str_has_prefix (mrl, prefix);
-}
-
-void
-totem_playlist_clear_with_prefix (TotemPlaylist *playlist, const char *prefix)
-{
-	totem_playlist_clear_with_compare (playlist,
-			(GCompareFunc) totem_playlist_compare_with_prefix,
-			(void *)prefix);
-}
-
 static int
 totem_playlist_compare_with_volume (gpointer a, gpointer b)
 {
 	GnomeVFSVolume *clear_volume = (GnomeVFSVolume *) b;
 	const char *mrl = (const char *) a;
 
-	char *filename;
 	GnomeVFSVolume *volume;
 	GnomeVFSVolumeMonitor *monitor;
 	gboolean retval = FALSE;
 
-	if (g_str_has_prefix (mrl, "file:///") == FALSE)
-		return FALSE;
-
 	monitor = gnome_vfs_get_volume_monitor ();
 
-	filename = g_filename_from_uri (mrl, NULL, NULL);
-	volume = gnome_vfs_volume_monitor_get_volume_for_path
-		(monitor, filename);
-	g_free (filename);
+	volume = totem_get_volume_for_media (mrl);
 
 	if (volume == clear_volume)
 		retval = TRUE;
 
-	gnome_vfs_volume_unref (volume);
+	if (volume != NULL)
+		gnome_vfs_volume_unref (volume);
 
 	return retval;
 }
 
 void
 totem_playlist_clear_with_gnome_vfs_volume (TotemPlaylist *playlist,
-		GnomeVFSVolume *volume)
+					    GnomeVFSVolume *volume)
 {
 	totem_playlist_clear_with_compare (playlist,
-			(GCompareFunc) totem_playlist_compare_with_volume,
-			volume);
+					   (GCompareFunc) totem_playlist_compare_with_volume,
+					   volume);
 }
 
 char *
@@ -1917,16 +1867,16 @@ totem_playlist_get_current_metadata (TotemPlaylist *playlist,
 		return FALSE;
 
 	gtk_tree_model_get_iter (playlist->_priv->model,
-			&iter,
-			playlist->_priv->current);
+				 &iter,
+				 playlist->_priv->current);
 
 	*artist = NULL;
 	gtk_tree_model_get (playlist->_priv->model,
-			&iter,
-			CACHE_ARTIST_COL, artist,
-			CACHE_TITLE_COL, title,
-			CACHE_ALBUM_COL, album,
-			-1);
+			    &iter,
+			    CACHE_ARTIST_COL, artist,
+			    CACHE_TITLE_COL, title,
+			    CACHE_ALBUM_COL, album,
+			    -1);
 
 	return (*artist != NULL);
 }
@@ -2062,6 +2012,25 @@ totem_playlist_set_playing (TotemPlaylist *playlist, TotemPlaylistStatus state)
 	return TRUE;
 }
 
+TotemPlaylistStatus
+totem_playlist_get_playing (TotemPlaylist *playlist)
+{
+	GtkTreeIter iter;
+	TotemPlaylistStatus status;
+
+	g_return_val_if_fail (TOTEM_IS_PLAYLIST (playlist), TOTEM_PLAYLIST_STATUS_NONE);
+
+	if (gtk_tree_model_get_iter (playlist->_priv->model, &iter, playlist->_priv->current) == FALSE)
+		return TOTEM_PLAYLIST_STATUS_NONE;
+
+	gtk_tree_model_get (playlist->_priv->model,
+			    &iter,
+			    PLAYING_COL, &status,
+			    -1);
+
+	return status;
+}
+
 void
 totem_playlist_set_previous (TotemPlaylist *playlist)
 {
